package com.ashokit.transaction.service;

import javax.management.RuntimeErrorException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.ashokit.transaction.model.Audit;
import com.ashokit.transaction.model.Employee;
import com.ashokit.transaction.model.Insurance;


@Service
public class OrganizationService {
	
	
	@Autowired
	private EmployeeService empService;
	
	
	@Autowired
	private InsuranceService insService;
	
	
	
	@Autowired
	private AuditService auditService;
	
	
	
	
	@Transactional(propagation = Propagation.REQUIRED)      //T1 Transaction Block
	//Default behaviour is required ==> Full commit/Full Rollback
	//Requires_new ==> new propgation level ==> partial commit/partial rollback
	public void onBoardEmployee(Employee empObj,Insurance insObj) {

		Employee employee = empService.saveEmployee(empObj); //This SQL will be moved to T1

		if (employee.getEmpId() != null) {
			auditService.saveAudit(new Audit("Employee creation is success for " + employee.getEmpName())); //T2 New Transaction
			insObj.setEmpId(employee.getEmpId()); //T2 will be saved to database 

		} else {
			auditService.saveAudit(new Audit("Employee creation is  Failed for " + employee.getEmpName())); //T2
		}

		insObj.setEmpId(empObj.getEmpId());
		insService.registerInsurance(insObj);   //This SQL moved to T1 Only
 
		if (insObj.getHealthInsuranceSchemeName().length() <=5) {
			
			auditService.saveAudit(new Audit("Insurance creation is Failed for  with insurance Name ....."
					+ insObj.getHealthInsuranceSchemeName() + "..... For the Employee " + employee.getEmpName())); //T3 New Tranaction
			throw new RuntimeErrorException(null, "Error in Insurance");
		}

		else {
			auditService.saveAudit(new Audit("Insurance creation is Sucess for " + employee.getEmpName())); //T3 New Tranaction
		}
	}

}
